/*
  Ices functions:
    IcesCommand()	   - get next command ( INIT / GET NEXT / SHUTDOWN )
    IcesGetPlaylistFile()  - get playlist file
    IcesGetRandomize()     - get 'randomize' switch
    IcesLog(str)           - output string to log
    IcesSetNext(lineno,	   - report next file name ( after 'GET NEXT' command )
                file
                [,metadata])

  This script will create a file .\etc\history.xml (that can be loaded in
  web browser).
*/


/* --- begin: Properties */

history._file = '.\etc\history.xml'
history._len = 10
history._enc = 'windows-1251'
history._xslt_file = 'history.xsl' /* without path! 
                                      (must be in history.xml directory) */

PL_REFRECH_WHEN_CHANGED = 'datetime' /* "datetime" or "size" */

/* --- end: Properties */


if history._enc = 'windows-1251' then
  do
    cp.history = ''
    cp.system  = '㪥뢠஫ᬨ'
  end
else
  do
    cp.history = ''
    cp.system  = ''
  end


call IcesLog 'Start'

playlist_stamp = -1
history.__pos = 0
history.0 = 0

call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

do forever
  cmd = IcesCommand()

  select
    when cmd = 'INIT' then
      do
        call IcesLog 'Initialization'
      end
    when cmd = 'GET NEXT' then
      do
        if RefreshPlaylist() | lineno = playlist.0 then
          lineno = 1
        else
          lineno = lineno + 1

        call IcesSetNext playlist.lineno.__line, playlist.lineno.__file
        call HistoryStore playlist.lineno.__file
      end
    when cmd = 'SHUTDOWN' then
      do
        call IcesLog 'Shutdown'
        leave
      end
    otherwise
      do
        call IcesLog 'Unknown command: ' || cmd
        exit(1)
      end
  end

end

EXIT(0)


RefreshPlaylist: procedure expose PL_REFRECH_WHEN_CHANGED playlist_stamp playlist.
  playlist_file = IcesGetPlaylistFile()
  randomize = IcesGetRandomize()

  if stream(playlist_file,'c','query exists') = "" then
  do
    call IcesLog 'Cannot open "' || playlist_file || '"'
    exit(1)
  end

  stamp = stream(playlist_file,'c','query ' || PL_REFRECH_WHEN_CHANGED)

  if stamp = playlist_stamp then
    return 0

  playlist_stamp = stamp

  drop playlist.

  i = 0
  do while lines(playlist_file)
    line = strip( linein(playlist_file) )
    if line = '' then
      iterate

    i = i+1
    playlist.i.__file = line
    playlist.i.__line = i
  end
  call stream playlist_file,'c','close'
  playlist.0 = i

  if i = 0 then
  do
    call IcesLog 'Playlist is empty'
    exit(1)
  end

  call IcesLog 'Playlist loaded, ' || playlist.0 || ' items'

  if randomize then
  do
    call random ,,time('S')

    do i = 1 to playlist.0
      item = random(1,playlist.0)

      temp = playlist.item.__file
      playlist.item.__file = playlist.i.__file
      playlist.i.__file = temp

      temp = playlist.item.__line
      playlist.item.__line = playlist.i.__line
      playlist.i.__line = temp
    end
  end
return 1

HistoryStore: procedure expose history. cp.
  file = arg(1)

  hpos = history.__pos + 1
  if hpos > history._len then hpos = 1
  history.__pos = hpos
  if history.0 < history._len then history.0 = history.0 + 1

  history.hpos.__file = filespec('name',file)
  history.hpos.__file = xml_str(translate(history.hpos.__file, cp.history, cp.system))
  history.hpos.__time = time()

  /* Read TAG */

  file_size = stream(file, 'c', 'query size')
  history.hpos.__tag = 0
  if file_size >= 1024 then
  do
    tag = charin(file, file_size-127, 128)
    if left(tag, 3) = 'TAG' then
    do
      history.hpos.__tag = 1
      history.hpos.__tag.__title	= xml_str( substr(tag, 4, 30) )
      history.hpos.__tag.__artist	= xml_str( substr(tag, 34, 30) )
      history.hpos.__tag.__album	= xml_str( substr(tag, 64, 30) )
      history.hpos.__tag.__year		= xml_str( substr(tag, 94, 4) )
      history.hpos.__tag.__comment	= xml_str( substr(tag, 98, 29) )
    end
  end
  call stream file, 'c', 'close'

  /* Update webfile */

  call SysFileDelete history._file
  j = hpos

  call lineout history._file, '<?xml version="1.0" encoding="'history._enc'"?>'
  call lineout history._file, '<?xml-stylesheet type="text/xsl" href="'history._xslt_file'"?>'
  call lineout history._file, '<icesStream>'

  do i = 1 to history.0
    call lineout history._file, '  <item id="'i'" time="'history.j.__time'">'
    call lineout history._file, '    <file>'history.j.__file'</file>'

    if history.j.__tag then
    do
      call lineout history._file, '    <title>'history.j.__tag.__title'</title>'
      call lineout history._file, '    <artist>'history.j.__tag.__artist'</artist>'
      call lineout history._file, '    <album>'history.j.__tag.__album'</album>'
      call lineout history._file, '    <year>'history.j.__tag.__year'</year>'
    end

    call lineout history._file, '  </item>'
    j = j - 1
    if j = 0 then j = history._len
  end

  call lineout history._file, '</icesStream>'
  call stream history._file, 'c', 'close'

return

xml_str: procedure
  str = strip(arg(1), 'T', '00'x)
  str = strip(str, , ' ')
  str = subset(str, '&', '&amp;')
  str = subset(str, '<', '&lt;')
  str = subset(str, '>', '&gt;')
return str

subset: procedure
  str = arg(1)
  s1 = arg(2)
  s2 = arg(3)

  s1_len = length(s1)
  s2_len = length(s2)

  p = 1
  do forever
    p = pos(s1,str,p)
    if p \= 0 then
    do
      str = left(str,p-1) || s2 || substr(str, p+s1_len)
      p = p+s2_len
    end
    else leave
  end
return str
